@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-"""API models for working with adapters and connections."""
+"""API for working with adapters."""
 import pathlib
 from typing import List, Optional, Union
 
@@ -7,15 +7,57 @@ from ...constants import CONFIG_TYPES, DEFAULT_NODE
 from ...exceptions import ApiError, NotFoundError
 from ...tools import path_read
 from ..mixins import ModelMixins
-from ..parsers.adapters import parse_adapters, parse_schema
-from ..parsers.config import config_build, config_unchanged, config_unknown
-from ..parsers.tables import tablize_adapters
+from ..parsers import (
+    config_build,
+    config_unchanged,
+    config_unknown,
+    parse_adapters,
+    parse_schema,
+    tablize_adapters,
+)
 from ..routers import API_VERSION, Router
 from .cnx import Cnx
 
 
 class Adapters(ModelMixins):
-    """API model for working with adapters."""
+    """API object for working with adapters.
+
+    Examples:
+        First, create a ``client`` using :obj:`axonius_api_client.connect.Connect`.
+
+        >>> # Get an adapter by name
+        >>> adapter = client.adapters.get_by_name(name="aws")
+        >>> print(adapter['status'])  # overall adapter status
+        >>> print(adapter['cnx_count_total'])  # total connection count
+        >>> print(adapter['cnx_count_broken'])  # broken connection count
+        >>> print(adapter['cnx_count_working'])  # working connection count
+        >>>
+        >>> # Get details of each connection of the adapter
+        >>> for cnx in adapter["cnx"]:
+        ...     print(cnx)
+        ...     print(cnx["working"])  # bool if connection is working or not
+        ...     print(cnx["error"])  # error from last fetch attempt
+        ...     print(cnx["config"])  # configuration of connection
+        ...     print(cnx["id"])  # ID of connection
+        ...     print(cnx["uuid"])  # UUID of connection
+        >>>
+        >>> # Update the generic advanced settings for the adapter
+        >>> updated_config = client.adapters.config_update(
+        ...     name="aws", last_seen_threshold_hours=24
+        ... )
+        >>>
+        >>> # Update the adapter specific advanced settings
+        >>> updated_config = client.adapters.config_update(
+        ...     name="aws", config_type="specific", fetch_s3=True
+        ... )
+        >>>
+        >>> # Update the discover advanced settings
+        >>> # XXX currently broken!
+        >>>
+        >>> # Upload a file for use in a connection later
+        >>> file_uuid = client.adapters.file_upload_path(path="test.csv")
+
+    """
 
     @property
     def router(self) -> Router:
@@ -23,13 +65,18 @@ class Adapters(ModelMixins):
         return API_VERSION.adapters
 
     def get(self) -> List[dict]:
-        """Pass."""
+        """Get all adapters on all nodes."""
         parsed = parse_adapters(raw=self._get())
         parsed = sorted(parsed, key=lambda x: [x["node_name"], x["name"]])
         return parsed
 
     def get_by_name(self, name: str, node: str = DEFAULT_NODE) -> dict:
-        """Pass."""
+        """Get an adapter by name on a single node.
+
+        Args:
+            name: name of adapter to get
+            node: name of node to get adapter from
+        """
         adapters = self.get()
 
         keys = ["node_name", "node_id"]
@@ -47,18 +94,24 @@ class Adapters(ModelMixins):
         err = f"No adapter named {name!r} found on node {node!r}"
         raise NotFoundError(tablize_adapters(adapters=adapters, err=err))
 
-    def config_get(
-        self,
-        name: str,
-        node: Optional[str] = DEFAULT_NODE,
-        config_type: Optional[str] = "generic",
-    ) -> dict:
-        """Pass."""
+    def config_get(self, name: str, node: str = DEFAULT_NODE, config_type: str = "generic") -> dict:
+        """Get the advanced settings for an adapter.
+
+        Args:
+            name: name of adapter to get advanced settings of
+            node: name of node to get adapter from
+            config_type: One of generic, specific, or discover
+        """
         adapter = self.get_by_name(name=name, node=node)
         return self.config_refetch(adapter=adapter, config_type=config_type)
 
     def config_refetch(self, adapter: dict, config_type: str = "generic") -> dict:
-        """Pass."""
+        """Re-fetch the advanced settings for an adapter.
+
+        Args:
+            adapter: adapter previously fetched from :meth:`get_by_name`
+            config_type: One of generic, specific, or discover
+        """
         schemas = adapter["schemas"]
         name_config = f"{config_type}_name"
         name_config = schemas.get(name_config)
@@ -78,7 +131,14 @@ class Adapters(ModelMixins):
     def config_update(
         self, name: str, node: str = DEFAULT_NODE, config_type: str = "generic", **kwargs
     ) -> dict:
-        """Pass."""
+        """Update the advanced settings for an adapter.
+
+        Args:
+            name: name of adapter to update advanced settings of
+            node: name of node to get adapter from
+            config_type: One of generic, specific, or discover
+            **kwargs: configuration to update advanced settings of config_type
+        """
         kwargs_config = kwargs.pop("kwargs_config", {})
         kwargs.update(kwargs_config)
         adapter = self.get_by_name(name=name, node=node)
@@ -117,28 +177,15 @@ class Adapters(ModelMixins):
         file_content_type: Optional[str] = None,
         node: str = DEFAULT_NODE,
     ) -> dict:
-        """Upload a str as a file to a specific adapter on a specific node.
+        """Upload a file to a specific adapter on a specific node.
 
         Args:
-            adapter (:obj:`str` or :obj:`dict`):
-
-                * If :obj:`str`, the name of the adapter to get the metadata for
-                * If :obj:`dict`, the metadata for a previously fetched adapter
-            field (:obj:`str`): name of field to store data in
-            name (:obj:`str`): filename to use when uploading file
-            content (:obj:`str` or :obj:`bytes`): content of file to upload
-            node (:obj:`str`, optional): default
-                :data:`.DEFAULT_NODE` -
-                node name running ``adapter``
-            content_type (:obj:`str`, optional): default ``None`` -
-                mime type of **content**
-
-        Returns:
-            :obj:`dict`: dict that can be used in a connection parameter of type
-            file that contains the keys:
-
-              * uuid: UUID of uploaded file
-              * filename: filename of uploaded file
+            name: name of adapter to upload file to
+            node: name of node to to upload file to
+            field_name: name of field (should match configuration schema key name)
+            file_name: name of file to upload
+            file_content: content of file to upload
+            file_content_type: mime type of file to upload
         """
         adapter = self.get_by_name(name=name, node=node)
 
@@ -155,26 +202,8 @@ class Adapters(ModelMixins):
         """Upload the contents of a file to a specific adapter on a specific node.
 
         Args:
-            adapter (:obj:`str` or :obj:`dict`):
-
-                * If :obj:`str`, the name of the adapter to get the metadata for
-                * If :obj:`dict`, the metadata for a previously fetched adapter
-            field (:obj:`str`): name of field to store data in
-            name (:obj:`str`): filename to use when uploading file
-            path (:obj:`str` or :obj:`pathlib.Path`): path to file containing contents
-                to upload
-            node (:obj:`str`, optional): default
-                :data:`.DEFAULT_NODE` -
-                node name running ``adapter``
-            content_type (:obj:`str`, optional): default ``None`` -
-                mime type of **content**
-
-        Returns:
-            :obj:`dict`: dict that can be used in a connection parameter of type
-            file that contains the keys:
-
-              * uuid: UUID of uploaded file
-              * filename: filename of uploaded file
+            path: path to file containing contents to upload
+            **kwargs: passed to :meth:`file_upload`
         """
         path, file_content = path_read(obj=path, binary=True, is_json=False)
         kwargs.setdefault("field_name", path.name)
@@ -184,15 +213,13 @@ class Adapters(ModelMixins):
 
     def _init(self, **kwargs):
         """Post init method for subclasses to use for extra setup."""
-        self.cnx = Cnx(parent=self)
+        self.cnx: Cnx = Cnx(parent=self)
+        """Work with adapter connections"""
+
         super(Adapters, self)._init(**kwargs)
 
     def _get(self) -> dict:
-        """Direct API method to get all adapters.
-
-        Returns:
-            :obj:`dict`: raw metadata for all adapters
-        """
+        """Direct API method to get all adapters."""
         path = self.router.root
         return self.request(method="get", path=path)
 
@@ -200,16 +227,13 @@ class Adapters(ModelMixins):
         """Direct API method to set advanced settings for an adapter.
 
         Args:
-            name_raw (:obj:`str`): unique plugin name of adapter to set
-                advanced settings for. unique plugin name refers to the name of
-                an adapter for a specific node
-            name_config (:obj:`str`): name of advanced settings to set -
-                either AdapterBase for generic advanced settings or
-                AwsSettings for adapter specific settings, if applicable
-            config (:obj:`dict`): the configuration values to set
+            name_raw: raw name of the adapter i.e. ``aws_adapter``
+            name_config: name of advanced settings to set
 
-        Returns:
-            :obj:`str`: empty str
+                * ``AdapterBase`` for generic advanced settings
+                * ``AwsSettings`` for adapter specific advanced settings (name changes per adapter)
+                * ``DiscoverySchema`` for discover advanced settings
+            new_config: the advanced configuration key value pairs to set
         """
         path = self.router.config_set.format(
             adapter_name_raw=name_raw, adapter_config_name=name_config
@@ -220,15 +244,12 @@ class Adapters(ModelMixins):
         """Direct API method to set advanced settings for an adapter.
 
         Args:
-            name_plugin (:obj:`str`): unique plugin name of adapter to set
-                advanced settings for. unique plugin name refers to the name of
-                an adapter for a specific node i.e. aws_adapter_0
-            name_config (:obj:`str`): name of advanced settings to set -
-                either AdapterBase for generic advanced settings or
-                AwsSettings for adapter specific settings, if applicable
+            name_plugin: plugin name of the adapter i.e. ``aws_adapter_0``
+            name_config: name of advanced settings to get
 
-        Returns:
-            :obj:`dict`: current settings for name_config of name_plugin
+                * ``AdapterBase`` for generic advanced settings
+                * ``AwsSettings`` for adapter specific advanced settings (name changes per adapter)
+                * ``DiscoverySchema`` for discover advanced settings
         """
         path = self.router.config_get.format(
             adapter_name_plugin=name_plugin, adapter_config_name=name_config
@@ -248,22 +269,13 @@ class Adapters(ModelMixins):
         """Direct API method to upload a file to a specific adapter on a specifc node.
 
         Args:
-            name_raw (:obj:`str`): name of the adapter to upload a file to
-            node_id (:obj:`str`): ID of node running **adapter**
-            field_name (:obj:`str`): name of setting this file is intended for
-            file_name (:obj:`str`): filename to use when uploading file
-            file_content (:obj:`str` or :obj:`bytes`): content to upload
-            file_content_type (:obj:`str`, optional): default ``None`` -
-                mime type of **content**
-            file_headers (:obj:`dict`, optional): default ``None`` -
-                headers to use for file **content**
-
-        Returns:
-            :obj:`dict`: dict that can be used in a connection parameter of type
-            file that contains the keys:
-
-              * uuid: UUID of uploaded file
-              * filename: filename of uploaded file
+            name_raw: raw name of the adapter i.e. ``aws_adapter``
+            node_id: ID of node running adapter
+            field_name: name of field (should match configuration schema key name)
+            file_name: name of file to upload
+            file_content: content of file to upload
+            file_content_type: mime type of file to upload
+            file_headers: headers to use for file
         """
         data = {"field_name": field_name}
         files = {"userfile": (file_name, file_content, file_content_type, file_headers)}
