@@ -1,12 +1,20 @@
 # -*- coding: utf-8 -*-
 """API models package."""
 from . import asset_mixin, devices, fields, labels, saved_query, users
+from .asset_mixin import AssetMixin
 from .devices import Devices
+from .fields import Fields
+from .labels import Labels
+from .saved_query import SavedQuery
 from .users import Users
 
 __all__ = (
     "Users",
     "Devices",
+    "AssetMixin",
+    "SavedQuery",
+    "Fields",
+    "Labels",
     "users",
     "devices",
     "fields",
