@@ -4,8 +4,7 @@ from typing import Optional
 
 from ...exceptions import ApiError, NotFoundError
 from ..mixins import ChildMixins
-from ..parsers.config import config_build, config_unchanged, config_unknown, parse_settings
-from ..parsers.tables import tablize
+from ..parsers import config_build, config_unchanged, config_unknown, parse_settings, tablize
 
 
 class SettingsChild(ChildMixins):
