@@ -18,6 +18,6 @@ def cmd(ctx, url, key, secret, **kwargs):
     client = ctx.obj.start_client(url=url, key=key, secret=secret)
 
     with ctx.obj.exc_wrap(wraperror=ctx.obj.wraperror):
-        data = client.system.discover.get()
+        data = client.dashboard.get()
 
     handle_export(ctx=ctx, data=data, **kwargs)
