@@ -26,26 +26,27 @@ CACHE: TTLCache = TTLCache(maxsize=1024, ttl=600)
 
 
 def get_field_vals(rows, field):
-    """Pass."""
+    """Test utility."""
     values = [x[field] for x in listify(rows) if x.get(field)]
     values = [x for y in values for x in listify(y)]
     return values
 
 
 def check_assets(rows):
-    """Pass."""
+    """Test utility."""
     assert isinstance(rows, list)
     for row in rows:
         check_asset(row)
 
 
 def check_asset(row):
-    """Pass."""
+    """Test utility."""
     assert isinstance(row, dict)
     assert row["internal_axon_id"]
 
 
 def exists_query(apiobj, fields=None, not_exist=False):
+    """Test utility."""
     if not fields:
         return None
 
@@ -65,6 +66,7 @@ def exists_query(apiobj, fields=None, not_exist=False):
 
 
 def get_schema(apiobj, field, key=None, adapter=AGG_ADAPTER_NAME):
+    """Test utility."""
     schemas = get_schemas(apiobj=apiobj, adapter=adapter)
     schema = apiobj.fields.get_field_schema(
         value=field,
@@ -74,14 +76,16 @@ def get_schema(apiobj, field, key=None, adapter=AGG_ADAPTER_NAME):
 
 
 def random_string(length):
+    """Test utility."""
     letters = string.ascii_lowercase
     result_str = "".join(random.choice(letters) for i in range(length))
     return result_str
 
 
-def get_rows_exist(apiobj, fields=None, max_rows=1, not_exist=False):
+def get_rows_exist(apiobj, fields=None, max_rows=1, not_exist=False, **kwargs):
+    """Test utility."""
     query = exists_query(apiobj=apiobj, fields=fields, not_exist=not_exist)
-    rows = apiobj.get(fields=fields, max_rows=max_rows, query=query)
+    rows = apiobj.get(fields=fields, max_rows=max_rows, query=query, **kwargs)
     if not rows:
         pytest.skip(f"No {apiobj} assets with fields {fields}")
     return rows[0] if max_rows == 1 else rows
@@ -89,6 +93,7 @@ def get_rows_exist(apiobj, fields=None, max_rows=1, not_exist=False):
 
 @cached(cache=CACHE)
 def get_schemas(apiobj, adapter=AGG_ADAPTER_NAME):
+    """Test utility."""
     return apiobj.fields.get()[adapter]
 
 
@@ -110,7 +115,7 @@ def log_check(caplog, entries, exists=True):
 
 #
 def get_cnx_existing(apiobj, name=None):
-    """Pass."""
+    """Test utility."""
     found = None
     adapters = apiobj.get()
     for adapter in adapters:
@@ -128,7 +133,7 @@ def get_cnx_existing(apiobj, name=None):
 
 
 def get_cnx_working(apiobj, name=None, reqkeys=None):
-    """Pass."""
+    """Test utility."""
     problem_children = [
         "symantec_altiris",  # AX-7165
     ]
@@ -159,7 +164,7 @@ def get_cnx_working(apiobj, name=None, reqkeys=None):
 
 
 def get_cnx_broken(apiobj, name=None):
-    """Pass."""
+    """Test utility."""
     found = None
     adapters = apiobj.get()
     for adapter in adapters:
@@ -178,19 +183,19 @@ def get_cnx_broken(apiobj, name=None):
 
 
 def get_url(request):
-    """Pass."""
+    """Test utility."""
     return request.config.getoption("--ax-url").rstrip("/")
 
 
 def get_key_creds(request):
-    """Pass."""
+    """Test utility."""
     key = request.config.getoption("--ax-key")
     secret = request.config.getoption("--ax-secret")
     return {"key": key, "secret": secret}
 
 
 def get_auth(request):
-    """Pass."""
+    """Test utility."""
     http = Http(url=get_url(request), certwarn=False)
 
     obj = auth.ApiKey(http=http, **get_key_creds(request))
@@ -199,7 +204,7 @@ def get_auth(request):
 
 
 def check_apiobj(authobj, apiobj):
-    """Pass."""
+    """Test utility."""
     url = authobj._http.url
     authclsname = format(authobj.__class__.__name__)
     assert authclsname in format(apiobj)
@@ -213,7 +218,7 @@ def check_apiobj(authobj, apiobj):
 
 
 def check_apiobj_children(apiobj, **kwargs):
-    """Pass."""
+    """Test utility."""
     for k, v in kwargs.items():
         attr = getattr(apiobj, k)
         attrclsname = format(attr.__class__.__name__)
@@ -230,7 +235,7 @@ def check_apiobj_children(apiobj, **kwargs):
 
 
 def check_apiobj_xref(apiobj, **kwargs):
-    """Pass."""
+    """Test utility."""
     for k, v in kwargs.items():
         attr = getattr(apiobj, k)
 
@@ -239,7 +244,7 @@ def check_apiobj_xref(apiobj, **kwargs):
 
 
 def load_clirunner(request, monkeypatch):
-    """Pass."""
+    """Test utility."""
     runner = CliRunner(mix_stderr=False)
 
     url = request.config.getoption("--ax-url")
@@ -253,7 +258,7 @@ def load_clirunner(request, monkeypatch):
 
 
 def check_stderr_lines(result):
-    """Pass."""
+    """Test utility."""
     stderr = result.stderr.splitlines()
 
     assert stderr[0].startswith("** WARNING: Unverified HTTPS request!")
@@ -261,27 +266,27 @@ def check_stderr_lines(result):
 
 
 class MockError(Exception):
-    """Pass."""
+    """Test utility."""
 
 
 class MockCtx:
-    """Pass."""
+    """Test utility."""
 
 
 def mock_failure(*args, **kwargs):
-    """Pass."""
+    """Test utility."""
     raise MockError("badwolf")
 
 
 def get_mockctx():
-    """Pass."""
+    """Test utility."""
     ctx = MockCtx()
     ctx.obj = Context()
     return ctx
 
 
 def check_csv_cols(content, cols):
-    """Pass."""
+    """Test utility."""
     QUOTING = csv.QUOTE_NONNUMERIC
     fh = StringIO()
     fh.write(content)
